/*-*-linux-c-*-*/

#include <stdlib.h>
#include <time.h> 
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>

#include <gtk/gtk.h>

#include "newtella.h"
#include "connection.h"
#include "gui.h"
#include "gnutella_header.h"
#include "hosts.h"
#include "downloads.h"

/* General functions for Download connections */

int download_write(gpointer gdata, gint s, GdkInputCondition cond)
{
	struct newtella_connection *d_con = (struct newtella_connection *)gdata;
	
	if (cond == GDK_INPUT_EXCEPTION) {
		d_con->state = GS_FAILED;
	        con_down_remove(d_con); 
		return -1;
	}
	
	if (con_send_from_queue(d_con) == -1) 
		gdk_input_remove(d_con->write_tag);
	/* else
		d_con->state = GS_FAILED;
	*/
	return 1;
}

int download_read(gpointer gdata, gint s, GdkInputCondition cond)
{
	struct newtella_connection *d_con = (struct newtella_connection *)gdata;
	int bytes_to_read, bytes_readed;
	gchar *seek;
	gchar *save_file = newtella_malloc(255);
	
	if (cond == GDK_INPUT_EXCEPTION) {
		d_con->state = GS_FAILED;
	        con_down_remove(d_con); 
		return -1;
	}
			
	bytes_to_read = bytes_avail(d_con->s);
	
	if (!bytes_to_read) {
		d_con->state = GS_FAILED;
		con_down_remove(d_con); 
		return -1;
	}

	d_con->b = d_con->buffer + d_con->buffer_len;
	bytes_readed = con_recv_packet(d_con, bytes_to_read);


        if (d_con->header_transmitted) {
		write(d_con->d_file, d_con->buffer, bytes_readed);
		con_reset_buffer(d_con);
		d_con->total_handled += bytes_readed;
		gui_update_down_con(d_con);
	} else {
		/* parse the http headers */
		do {
			if (!memcmp("GIV", d_con->buffer, 3)) {
				strcpy(save_file, g_strdup(gl_options->download_dir));
				strcat(save_file, d_con->fi->file_name);
			        
				d_con->d_file = open(save_file, O_WRONLY | O_CREAT | O_TRUNC, 0644);
				d_con->header_transmitted = 1;
				d_con->state = GS_CONNECTED;
				
				break;
			}

			seek = (gchar *) memchr(d_con->buffer, '\n', bytes_readed);
			
			if (!seek)
				return 1;
			if (*(seek-1) == '\r')
				*(seek-1) = 0;
			else {
				printf("Wrong HTTP Header\n");
				con_down_remove(d_con);
				return -1;
			} 
			
			*seek++ = 0;
		
			/* check for HTTP messages */
			if (!g_strncasecmp(d_con->buffer, "HTTP/1.0 200", 12) || 
			    !g_strncasecmp(d_con->buffer, "HTTP 200", 8))
				d_con->download_ok = 1;
				
			if (!g_strncasecmp(d_con->buffer, "HTTP/1.0 404", 12) 
			    || !g_strncasecmp(d_con->buffer, "HTTP 404", 8)) {
				printf("Error in download\n");
				con_down_remove(d_con);
				return -1;
			}

			if (!g_strncasecmp(seek, "Content-length:", 15)) {
				/* get the size */
				guint32 sz = atol(seek + 15);
				
				if (sz != d_con->fi->size) { 
					con_down_remove(d_con);
					return -1;
				}

				d_con->header_transmitted = 1;
				d_con->state = GS_CONNECTED;
			
				strcpy(save_file, g_strdup(gl_options->download_dir));
				strcat(save_file, d_con->fi->file_name);
			       
				d_con->d_file = open(save_file, O_WRONLY | O_CREAT | O_TRUNC, 0644);
				seek = memchr(d_con->buffer, '\n', bytes_readed);

				d_con->buffer_len = seek - d_con->buffer;

				write(d_con->d_file, seek, bytes_readed-d_con->buffer_len);

				con_reset_buffer(d_con);
			}
				
		} while (!d_con->header_transmitted);
	}
		
	return 1;
}

int download_accept(gpointer data, gint source, GdkInputCondition cond)
{
	struct sockaddr_in addr;
	size_t len;
	struct newtella_connection *d_con =  data;
	int rval = accept(d_con->s, (struct sockaddr_in *) &addr, &len);

	if (rval == -1)
		printf("Can't accept\n");
		
	d_con->s = rval;

	gdk_input_remove(d_con->read_tag);
	d_con->read_tag = gdk_input_add(d_con->s, GDK_INPUT_READ | GDK_INPUT_EXCEPTION, 
					(GdkInputFunction) download_read, (gpointer)d_con);
	return 1;
}

int download_add(struct file_found *download_file)
{
	struct newtella_connection *d_con = newtella_malloc(sizeof(struct newtella_connection));

	d_con->fi = download_file;
	d_con->ip = gl_my_host_info->ip;
	d_con->packet_queue = NULL;
	d_con->queue_size = 0;
	d_con->buffer_len = 0;
	d_con->header_transmitted = 0;
	d_con->download_ok = 0;
	d_con->id = ++gl_options->current_down_con;
	d_con->total_handled = 0;

	if (gl_options->current_down_con == 1)
		gl_down_con = g_list_append(NULL, d_con);
	else
		gl_down_con = g_list_append(gl_down_con, d_con);

	download_start(d_con);
	
	return 1;
}

int download_start(struct newtella_connection *d_con)
{
	struct sockaddr_in gnode, addr;
	size_t option;
	char http_header[4096];
	
	if (!host_is_private(htonl(d_con->fi->ip), d_con->fi->port)) {
		/* direct connection */
		d_con->state = GS_WAITINGCONNECT;
		d_con->s = newtella_connect(d_con->fi->ip, d_con->fi->port);

			
		setsockopt(d_con->s, SOL_SOCKET, SO_KEEPALIVE, (void *) &option, sizeof(option));
		setsockopt(d_con->s, SOL_SOCKET, SO_REUSEADDR, (void *) &option, sizeof(option));
		
		
		d_con->read_tag = gdk_input_add(d_con->s, GDK_INPUT_READ | GDK_INPUT_EXCEPTION, 
			      (GdkInputFunction) download_read, (gpointer)d_con);
	
		d_con->write_tag = gdk_input_add(d_con->s, GDK_INPUT_WRITE | GDK_INPUT_EXCEPTION, 
			      (GdkInputFunction) download_write, (gpointer)d_con);
			      
		if (d_con->s == -1) {
			d_con->state = GS_FAILED;
			return (-1);
		}

		g_snprintf(http_header, sizeof(http_header),
		"GET /get/%i/%s HTTP/1.0\r\nConnection: Keep-Alive\r\nRange: bytes=0-\r\n\r\n",
		d_con->fi->index, d_con->fi->file_name);
		
		con_send_packet(d_con, http_header, sizeof(http_header));

	} else {
		/* firewall - send a push */
		d_con->state = GS_WAITINGPUSH;
		d_con->s = socket(AF_INET, SOCK_STREAM, 0);
	
		if (fcntl(d_con->s, F_SETFL, O_NONBLOCK) == -1) {
			perror("fcntl");
			exit(EXIT_FAILURE);
		}

		gnode.sin_family = AF_INET;
		gnode.sin_addr.s_addr = htonl(INADDR_ANY);
		gnode.sin_port = htons(0);

		bind(d_con->s, (struct sockaddr_in *)&gnode, sizeof(gnode));

		if (listen(d_con->s, 1) == -1)
			printf("Can't listen\n");
		
		/* get the port number */
		option = sizeof(struct sockaddr_in);
	
		if (getsockname(d_con->s, (struct sockaddr_in *) &addr, &option) == -1)
        		printf("Can't getsockname\n");
		
		d_con->port = ntohs(addr.sin_port);
	
		gnutella_send_push(d_con);
	
		d_con->read_tag = gdk_input_add(d_con->s, GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
					(GdkInputFunction) download_accept, (gpointer) d_con);
	}
	
	gui_add_down_con_to_list(d_con);
	/* update time */
	d_con->last_time = time((time_t)NULL);
	
	return 1;
}




