/*-*-linux-c-*-*/

/*
 *  gnewtellium - Newtella for Unix
 *  Copyright (C) 2001 Elias Athanasopoulos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <netinet/in.h>
#include <string.h>

#include "newtella.h"

#include "guid.h"
#include "connection.h"
#include "gnutella_header.h"
#include "global.h"
#include "file.h"

extern struct host_info *gl_my_host_info;
extern struct newtella_options *gl_options;

int gnutella_send_ping(struct newtella_connection *con)
{
	struct gnutella_header *header = newtella_malloc(GN_HEADER_SIZE);
	char *data;

	header->_guid = getguid();
	header->function = GNUTELLA_INIT;
	header->TTL = gl_options->ping_TTL;
	header->hops = 0;
	header->length = 0;

	data = newtella_malloc(GN_HEADER_SIZE);

	memcpy(data, &(header->_guid), 16);
	memcpy(data+16, &(header->function), 1);
	memcpy(data+17, &(header->TTL), 1);
       	memcpy(data+18, &(header->hops), 1);
	memcpy(data+19, &(header->length), 4);

	con_send_packet(con, data, GN_HEADER_SIZE);
	
	newtella_free (header);
	newtella_free (data);

	return 1;
}

int gnutella_send_pong(struct newtella_connection *con, guid _guid)
{
	struct gnutella_header *header;
	struct gnutella_pong_header *gpong;
	guint32 pack_size = 0;
	char *data;

	header = newtella_malloc(GN_HEADER_SIZE);
	gpong = newtella_malloc(GN_PONG_SIZE);
       
	header->_guid = _guid;
	header->function = GNUTELLA_INITRESPONSE;
	header->hops = 0;
	header->length = GN_PONG_SIZE;

	gpong->host_ip = gl_my_host_info->ip;
	gpong->host_port = ntohs(gl_my_host_info->port);
	gpong->file_count = gl_my_host_info->file_count;
	gpong->files_total_size = gl_my_host_info->files_total_size;

	pack_size = GN_HEADER_SIZE + header->length;

	data = newtella_malloc(pack_size);
	
	memcpy(data, &(header->_guid), 16);
	memcpy(data+16, &(header->function), 1);
	memcpy(data+17, &(header->TTL), 1);
       	memcpy(data+18, &(header->hops), 1);
	memcpy(data+19, &(header->length), 4);
	
	data += GN_HEADER_SIZE;

	memcpy(data, &(gpong->host_port), 2);
	memcpy(data+2, &(gpong->host_ip), 4);
	memcpy(data+6, &(gpong->file_count), 4);
	memcpy(data+10, &(gpong->files_total_size), 4);

	data -= GN_HEADER_SIZE;

        con_send_packet(con, data, pack_size);
       	
	newtella_free (header);
	newtella_free (gpong);
	newtella_free (data);

	return 1;
}

int gnutella_send_query(char *str)
{
	struct gnutella_header *header;
	struct gnutella_query_header *query_header;
	long pack_size = 0;
	char *data;

        query_header = newtella_malloc(GN_QUERY_SIZE);
	query_header->min_speed = 0;

	header = newtella_malloc(GN_HEADER_SIZE);
	header->_guid = getguid();
	header->function = GNUTELLA_SEARCH;
	header->TTL = gl_options->search_TTL;
	header->hops = 0;
	header->length = GN_QUERY_SIZE + strlen(str) + 1;	

	pack_size = GN_HEADER_SIZE + header->length;

	data = newtella_malloc(pack_size);
  
	memcpy(data, &(header->_guid), 16);
	memcpy(data+16, &(header->function), 1);
	memcpy(data+17, &(header->TTL), 1);
       	memcpy(data+18, &(header->hops), 1);
	memcpy(data+19, &(header->length), 4);
 
	data += GN_HEADER_SIZE;
	memcpy(data, query_header, GN_QUERY_SIZE);
	data += GN_QUERY_SIZE;
	strcpy(data, str);

	data -= GN_HEADER_SIZE + GN_QUERY_SIZE;
	con_send_to_all(data, pack_size);
	
	newtella_free(header);
	newtella_free(query_header);
	newtella_free(data);

	return 1;
}

int gnutella_send_query_response(struct newtella_connection *con, GList *found_hits, guid _guid)
{
	GList *iter;
	struct gnutella_hits *ghits = newtella_malloc(sizeof(struct gnutella_hits));
	struct gnutella_header *header = newtella_malloc(sizeof(struct gnutella_header));
	struct file_entry *fe;
	char *data;
	int i = 0, str_offs = 0, packet_size = 0;

	ghits->hits = g_list_length(found_hits);
	ghits->port = gl_my_host_info->port;
	ghits->ip = htonl(gl_options->local_ip);
	ghits->speed = gl_options->con_speed;

	header->_guid = _guid;
	header->function = GNUTELLA_SEARCHRESPONSE;
	header->TTL = gl_options->search_TTL;
	header->hops = 0;

	/* find out the size of the packet 
	 * we have to add: 
	 * - gnutella header
	 * - hits header
	 * - (hit header) x number of hits
	 * - size of each returned file name + 2 for \n\n
	 * - our guid (last 16 bytes)
	 */
	packet_size = GN_HEADER_SIZE + GN_HITS_SIZE + ghits->hits*GN_HIT_SIZE + 16;

	iter = g_list_first(found_hits);
	while (iter) {
		fe = (struct file_entry*)iter->data;
		/* add size of the file name to packet size */
		packet_size += strlen(fe->name) + 2;
		
		iter = g_list_next(iter);
	}

        /* now we know exactly how 
	 * many bytes are needed for
	 * the new packet
	 */
	data = newtella_malloc(packet_size);
       
	/* copy standard header */
	memcpy(data, &(header->_guid), 16);
	memcpy(data+16, &(header->function), 1);
	memcpy(data+17, &(header->TTL), 1);
       	memcpy(data+18, &(header->hops), 1);

	/* copy standard gnutella hits header */
	memcpy(data+GN_HEADER_SIZE, &(ghits->hits), 1);
	memcpy(data+GN_HEADER_SIZE+1, &(ghits->port), 2);
	memcpy(data+GN_HEADER_SIZE+3, &(ghits->ip), 4);
	memcpy(data+GN_HEADER_SIZE+7, &(ghits->speed), 4);
	

	/* copy each hit */
	iter = g_list_first(found_hits);
	while (iter) {
		fe = (struct file_entry*)iter->data;
		
		memcpy(data+str_offs+GN_HEADER_SIZE+GN_HITS_SIZE+GN_HIT_SIZE*i, &(fe->index), 4);
		memcpy(data+str_offs+GN_HEADER_SIZE+GN_HITS_SIZE+GN_HIT_SIZE*i+4, &(fe->size), 4);
		memcpy(data+str_offs+GN_HEADER_SIZE+GN_HITS_SIZE+GN_HIT_SIZE*i+8, fe->name, strlen(fe->name)+2);
		i++;
		str_offs += strlen(fe->name)+2;
		iter = g_list_next(iter);
	}

	  
	/* copy guid to the last 16 bytes */
	memcpy(data+packet_size-16, &(gl_options->gl_guid), 16);
	header->length = packet_size - GN_HEADER_SIZE;
	
	/* adjust header length */
	memcpy(data+19, &(header->length), 4);

	con_send_packet(con, data, header->length+GN_HEADER_SIZE);
	
	newtella_free(ghits);
	newtella_free(data);

	return 1;
}

int gnutella_send_push(struct newtella_connection *d_con)
{
	struct gnutella_header *header = newtella_malloc(GN_HEADER_SIZE);
	struct gnutella_push *gpush = newtella_malloc(GN_PUSH_SIZE);
	char *data;

	header->_guid = getguid();
	header->function = GNUTELLA_PUSHREQUEST;
	header->TTL = gl_options->ping_TTL;
	header->hops = 0;
	header->length = GN_PUSH_SIZE;

	gpush->_guid = d_con->fi->_guid;
	gpush->index = d_con->fi->index;
	gpush->ip = htonl(gl_options->local_ip);
	gpush->port = d_con->port;

	data = newtella_malloc(GN_HEADER_SIZE+GN_PUSH_SIZE);
	memcpy(data, &(header->_guid), 16);
	memcpy(data+16, &(header->function), 1);
	memcpy(data+17, &(header->TTL), 1);
       	memcpy(data+18, &(header->hops), 1);
	
	data += GN_HEADER_SIZE;

	memcpy(data, &(gpush->_guid), 16);
	memcpy(data+16, &(gpush->index), 4);
	memcpy(data+20, &(gpush->ip), 4);
       	memcpy(data+24, &(gpush->port), 2);
	
	data -= GN_HEADER_SIZE;
      
	con_send_to_all(data, GN_HEADER_SIZE+GN_PUSH_SIZE);

	newtella_free (header);
	newtella_free (gpush);
	newtella_free (data);

	return 1;
}

int gnutella_fwd_ping(struct gnutella_header *gh, struct newtella_connection *s_con)
{
	GList *iter;
	struct newtella_connection *con;
	char *data = newtella_malloc(GN_HEADER_SIZE);	

	memcpy(data, &(gh->_guid), 16);
	memcpy(data+16, &(gh->function), 1);
	memcpy(data+17, &(gh->TTL)-1, 1);
	memcpy(data+18, &(gh->hops)+1, 1);
	memcpy(data+19, &(gh->length), 4);

	/* fwd ping to all connections except the one which sent it */
	iter = g_list_first(gl_con);
	while (iter != NULL) {
		con = (struct newtella_connection *) iter->data;
		
		if (!con) 
			break;

		if ((con->id != s_con->id) && (con->state == GS_CONNECTED))
			con_send_packet(con, data, GN_HEADER_SIZE);
		
		iter = g_list_next(iter);
	}
	
	newtella_free (data);

	return 1;
}

int gnutella_fwd_pong(struct newtella_connection *con,
		      struct gnutella_header *gh, struct gnutella_pong_header *gpong)
{
	char *data = newtella_malloc(GN_HEADER_SIZE + GN_PONG_SIZE);
			
	memcpy(data, &(gh->_guid), 16);
	memcpy(data+16, &(gh->function), 1);
	memcpy(data+17, &(gh->TTL)-1, 1);
	memcpy(data+18, &(gh->hops)+1, 1);
	memcpy(data+19, &(gh->length), 4);
	
	data += GN_HEADER_SIZE;
			
	memcpy(data, &(gpong->host_port), 2);
	memcpy(data+2, &(gpong->host_ip), 4);
	memcpy(data+6, &(gpong->file_count), 4);
	memcpy(data+10, &(gpong->files_total_size), 4);
	
	data -= GN_HEADER_SIZE;
	con_send_packet(con, data, GN_HEADER_SIZE+GN_PONG_SIZE);

	newtella_free (data);

	return 1;
}

int gnutella_fwd_query(struct gnutella_header *gh, struct gnutella_query_header *gquery,
		       struct newtella_connection *s_con)
{
	char *data = newtella_malloc(GN_HEADER_SIZE+GN_QUERY_SIZE+strlen(gquery->name)+1);
	struct newtella_connection *con;
	GList *iter;

	memcpy(data, &(gh->_guid), 16);
	memcpy(data+16, &(gh->function), 1);
	memcpy(data+17, &(gh->TTL)-1, 1);
       	memcpy(data+18, &(gh->hops)+1, 1);
	memcpy(data+19, &(gh->length), 4);
 
	data += GN_HEADER_SIZE;
	memcpy(data, gquery, GN_QUERY_SIZE);
	data += GN_QUERY_SIZE;
	strcpy(data, gquery->name);
	
	data -= GN_HEADER_SIZE + GN_QUERY_SIZE;
	
        /* fwd query to all connections except the one which sent it */
	iter = g_list_first(gl_con);
	while (iter != NULL) {
		con = (struct newtella_connection *) iter->data;
		
		if (!con) 
			break;

		if ((con->id != s_con->id) && (con->state == GS_CONNECTED))
			con_send_packet(con, data, 
					GN_HEADER_SIZE+GN_QUERY_SIZE+strlen(gquery->name)+1);
		
		iter = g_list_next(iter);
	}
    

	newtella_free (data);

	return 1;
}

int serialize_header(struct gnutella_header *hdr, char *buf)
{
       
	memcpy(&(hdr->_guid), buf, 16);
	memcpy(&(hdr->function), buf+16, 1);
	memcpy(&(hdr->TTL), buf+17, 1);
	memcpy(&(hdr->hops), buf+18, 1);
	memcpy(&(hdr->length), buf+19, 4);
	
#ifdef SOLARIS
	/*FIXME: Should be #ifdef 64BITOS or something*/
	hdr->length = GUINT32_TO_LE(hdr->length);
#endif
	return 1;
}

int serialize_pong(struct gnutella_pong_header *gpong, char *buf)
{
	memcpy(&(gpong->host_port), buf, 2);
	memcpy(&(gpong->host_ip), buf+2, 4);
	memcpy(&(gpong->file_count), buf+6, 4);
	memcpy(&(gpong->files_total_size), buf+10, 4);
	
#ifdef SOLARIS
	/*FIXME:  This should be #ifdef 64BITARCH instead
	I Must make a note here because I don't know why this
	was needed.  It took me a while to track this down.
	My guess is that the protocol forgot to mention port
	is little-endian.*/
	gpong->host_port = GUINT16_TO_LE(gpong->host_port);
#endif

	return 1;
}

int serialize_search(struct gnutella_query_header *gquery, char *buf)
{

	memcpy(&(gquery->min_speed), buf, 2);
	/*gquery->name = newtella_malloc(strlen(p));
	  strcpy(gquery->name, p);*/

	return 1;
}

int serialize_hit(struct gnutella_hit *ghit, char *buf)
{
	char *p = (buf + 8);

	memcpy(&(ghit->index), buf, 4);
	memcpy(&(ghit->size), buf+4, 4);
	ghit->name = newtella_malloc(strlen(p)+1);
	strcpy(ghit->name, p);

	return 1;
}

int serialize_hits(struct gnutella_hits *ghits, char *buf)
{
	memcpy(&(ghits->hits), buf, 1);
	memcpy(&(ghits->port), buf+1, 2);
	memcpy(&(ghits->ip), buf+3, 4);
	memcpy(&(ghits->speed), buf+7, 4);

	return 1;
}

int serialize_push(struct gnutella_push *gpush, char *buf)
{
	memcpy(&(gpush->_guid), buf, 16);
	memcpy(&(gpush->index), buf+16, 4);
	memcpy(&(gpush->ip), buf+20, 4);
	memcpy(&(gpush->port), buf+24, 2);
	
	return 1;
}



