/*-*-linux-c-*-*/

/*
 *  gnewtellium - Newtella for Unix
 *  Copyright (C) 2001 Elias Athanasopoulos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef GNUTELLA_HEADER_H
#define GNUTELLA_HEADER_H

#include "guid.h"
#include "connection.h"

typedef char HASH[20];

typedef union 
{
	char chars[4];
	guint32 dword;
} IP;

typedef enum {
        GNUTELLA_INIT=0x00, 
	GNUTELLA_INITRESPONSE=0x01, 
	GNUTELLA_PUSHREQUEST=0x40, 
	GNUTELLA_SEARCH=0x80, 
	GNUTELLA_SEARCHRESPONSE=0x81
} gnutella_function;

struct gnutella_header {
	guid _guid;
	guint8 function;
	guint8 TTL;
	guint8 hops;
	guint32 length; 
};
#define GN_HEADER_SIZE 23

struct gnutella_pong_header {
	guint16 host_port;
	guint32 host_ip;
	guint32 file_count;
	guint32 files_total_size;
};
#define GN_PONG_SIZE 14

struct gnutella_query_header {
	guint16 min_speed;
	guint8 *name;
};
#define GN_QUERY_SIZE 2

struct gnutella_hit {
	guint32 index;
	guint32 size;
	guint8 *name;
};
#define GN_HIT_SIZE 8
 
struct gnutella_hits {
	guint8 hits;
	guint16 port;
	guint32 ip;
	guint32 speed;
	GList *rset;
};
#define GN_HITS_SIZE 11

struct gnutella_push {
	guid _guid;
	guint32 index;
	guint32 ip;
	guint16 port;
};
#define GN_PUSH_SIZE 26

int gnutella_send_ping(struct newtella_connection *con);
int gnutella_send_pong(struct newtella_connection *con, guid _guid);
int gnutella_send_query(char *str);
int gnutella_send_push(struct newtella_connection *d_con);
int gnutella_send_query_response(struct newtella_connection *con, GList *found_hits, guid _guid);

int gnutella_fwd_ping(struct gnutella_header *gh, struct newtella_connection *s_con);
int gnutella_fwd_pong(struct newtella_connection *con,
		      struct gnutella_header *gh, struct gnutella_pong_header *gpong);
int gnutella_fwd_query(struct gnutella_header *gh, struct gnutella_query_header *gquery,
		       struct newtella_connection *s_con);

int serialize_header(struct gnutella_header *hdr, char *buf);
int serialize_pong(struct gnutella_pong_header *gpong, char *buf);
int serialize_search(struct gnutella_query_header *gquery, char *buf);
int serialize_hit(struct gnutella_hit *ghit, char *buf);
int serialize_hits(struct gnutella_hits *ghits, char *buf);
int serialize_push(struct gnutella_push *gpush, char *buf);

#endif




