/*-*-linux-c-*-*/

/*
 *  gnewtellium - Newtella for Unix
 *  Copyright (C) 2001 Elias Athanasopoulos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gtk/gtk.h>

#include <stdlib.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>

#include "newtella.h"
#include "global.h"
#include "gui.h"
#include "gnutella_header.h"
#include "config.h"
#include "file.h"
#include "downloads.h"

void gui_dump_con(void)
{
	gchar l_ip[255], l_status[255], l_sent[255], l_recv[255];

	int row = 0;
	GList *iter;
	struct newtella_connection *con;

	
	gl_con = g_list_first(gl_con);
	iter = gl_con;
  
	while (iter != NULL) {
		con = gl_con->data;

		g_snprintf(l_ip, sizeof(l_ip), "%s:%d", ip2str(htonl(con->ip)), con->port);
	       
		switch (con->state) {
		case GS_WAITINGCONNECT:
			strcpy(l_status, "Connecting...");
			break;
		case GS_CONNECTED:
			strcpy(l_status, "OK");
			break; 
		case GS_FAILED:
			strcpy(l_status, "Failed...");
			break; 
		case GS_TIMEOUT:
			strcpy(l_status, "TIME OUT");
			break;
		default:
			break;
		}
		
		g_snprintf(l_sent, sizeof(l_sent), "%d/%d", con->total_sent, 
			   con->total_dropped_to_send);
		
		g_snprintf(l_recv, sizeof(l_recv), "%d/%d/%d", con->total_handled, 
			   con->total_received_dropped, con->total_bad_received);
		
		gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 0, l_ip);
		gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 1, l_status);
		gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 2, l_sent);
		gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 3, l_recv);
		
		row++;

		iter = g_list_next(gl_con);
		if (iter != NULL)
			gl_con = iter;
	}

}

void gui_update_con_status(struct newtella_connection *con)
{
	switch (con->state) {
		case GS_WAITINGCONNECT:
			gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 1, "Connecting...");
			break;
		case GS_CONNECTED:
			gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 1, "OK");
			break; 
		case GS_FAILED:
			gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 1, "Failed");
			break; 
		case GS_TIMEOUT:
			gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 1, "Time Out");
			break;
		default:
			break;
	}
}

void gui_update_found_results(int id)
{
	GList *iter;
	struct file_found *found;
	gchar buf[255], l_size[255], l_speed[255], l_loc[255] = "", l_recv[255] = "";
	int row = id;

	iter = g_list_nth(gl_results, row);
	
	while (iter != NULL) {
		found = iter->data;

		if (found->size < 1024) 
			g_snprintf(l_size, sizeof(l_size), "%u Bytes", found->size);
		else if (found->size < 1048576) 
			g_snprintf(l_size, sizeof(l_size), "%.1fKB", (float)found->size/1024.0);
		else if (found->size < 1073741824) 
			g_snprintf(l_size, sizeof(l_size), "%.1fMB", (float)found->size/1048576.0);
		else 
			g_snprintf(l_size, sizeof(l_size), "%.1fGB", (float)found->size/1073741824.0);

		g_snprintf(l_speed, sizeof(l_speed), "%d", found->speed);
		g_snprintf(l_loc, sizeof(l_loc), "%s", ip2str(htonl(found->ip)));

		gtk_clist_set_text(GTK_CLIST(search_list), row, 0, found->file_name);
		gtk_clist_set_text(GTK_CLIST(search_list), row, 1, l_size);
		gtk_clist_set_text(GTK_CLIST(search_list), row, 2, l_speed);
		gtk_clist_set_text(GTK_CLIST(search_list), row, 3, l_loc);
		gtk_clist_set_text(GTK_CLIST(search_list), row, 4, l_recv);

		row++;
		
		iter = g_list_next(iter);
	}

	/* update the found files label */
	g_snprintf(buf, sizeof(buf), "%d", gl_options->files_found);
	gtk_label_set_text(GTK_LABEL(files_found_label), buf);
}

void gui_clean_found_results(void)
{
	GList *iter;

	iter = g_list_first(gl_results);
	while (iter) {
		newtella_free(iter->data);

		iter = g_list_next(iter);
	}

	g_list_free(gl_results);

	gtk_clist_clear(GTK_CLIST(search_list));
	gtk_label_set_text(GTK_LABEL(files_found_label), "0");

	gl_options->files_found = 0;
	gl_results = NULL;
}

void gui_add_con_to_list(struct newtella_connection *con)
{
	gchar *dummy[4] = { "", "", "", "" };
	gchar l_ip[255];

	gtk_clist_append(GTK_CLIST(con_list), dummy);

	g_snprintf(l_ip, sizeof(l_ip), "%s:%d", ip2str(htonl(con->ip)), con->port);
	gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 0, l_ip);
	gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 2, "0/0");
	gtk_clist_set_text(GTK_CLIST(con_list), con->id-1, 3, "0/0/0");
		
	gui_update_con_status(con);

}

void gui_add_down_con_to_list(struct newtella_connection *d_con)
{
	gchar *dummy[5] = { "", "", "", "", "" };
	gchar l_size[255], l_status[255], l_speed[255], l_server[255];
	
	gtk_clist_append(GTK_CLIST(download_list), dummy);
	
	if (d_con->fi->size < 1024) 
		g_snprintf(l_size, sizeof(l_size), "%u Bytes", d_con->fi->size);
	else if (d_con->fi->size < 1048576) 
		g_snprintf(l_size, sizeof(l_size), "%.1fKB", (float)d_con->fi->size/1024.0);
	else if (d_con->fi->size < 1073741824) 
		g_snprintf(l_size, sizeof(l_size), "%.1fMB", (float)d_con->fi->size/1048576.0);
	else  
		g_snprintf(l_size, sizeof(l_size), "%.1fGB", (float)d_con->fi->size/1073741824.0);
	
	switch (d_con->state) {
	case GS_WAITINGCONNECT:
		strcpy(l_status, "Connecting...");
		break;
	case GS_WAITINGPUSH:	
		strcpy(l_status, "Push sent");
		break;
	case GS_FAILED:
		strcpy(l_status, "Failed");
		break;
	default:
		break;
	}	
	
	g_snprintf(l_speed, sizeof(l_speed), "%d", d_con->fi->speed);
	g_snprintf(l_server, sizeof(l_server), "%s:%d", ip2str(htonl(d_con->fi->ip)), d_con->fi->port);

	gtk_clist_set_text(GTK_CLIST(download_list), d_con->id-1, 0, d_con->fi->file_name);
	gtk_clist_set_text(GTK_CLIST(download_list), d_con->id-1, 1, l_size);
	gtk_clist_set_text(GTK_CLIST(download_list), d_con->id-1, 2, l_status);
	gtk_clist_set_text(GTK_CLIST(download_list), d_con->id-1, 3, l_speed);
	gtk_clist_set_text(GTK_CLIST(download_list), d_con->id-1, 4, l_server);
 
	/* go to downloads */
	gtk_notebook_set_page(GTK_NOTEBOOK(main_notebook), 1);
}

void gui_update_down_con(struct newtella_connection *d_con)
{
	gchar l_status[255], l_speed[255];
	
	switch (d_con->state) {
	case GS_WAITINGCONNECT:
		strcpy(l_status, "Connecting...");
		break;
	case GS_WAITINGPUSH:	
		strcpy(l_status, "Push sent");
		break;
	case GS_FAILED:
		strcpy(l_status, "Failed");
		break;
	case GS_CONNECTED:
		g_snprintf(l_speed, sizeof(l_speed), "%.1f k/s",
			   (float) (d_con->total_handled/(float)(time((time_t *)NULL)-d_con->last_time)/1000.0));
		g_snprintf(l_status, sizeof(l_status), "%.1f %%", 
			   (float) (d_con->total_handled/(float)d_con->fi->size)*100.0);
		break;
	default:
		break;
	}	

	gtk_clist_set_text(GTK_CLIST(download_list), d_con->id-1, 2, l_status);
}


void gui_add_up_con_to_list(struct newtella_connection *u_con, char *file_name)
{
	gchar *dummy[5] = { "", "", "", "", "" };
	gchar l_size[255], l_status[255], /*l_speed[255], */l_server[255];
	
	gtk_clist_append(GTK_CLIST(upload_list), dummy);
	
	if (u_con->u_file_size < 1024) 
		g_snprintf(l_size, sizeof(l_size), "%u Bytes", u_con->u_file_size);
	else if (u_con->u_file_size < 1048576) 
		g_snprintf(l_size, sizeof(l_size), "%.1fKB", (float)u_con->u_file_size/1024.0);
	else if (u_con->u_file_size < 1073741824) 
		g_snprintf(l_size, sizeof(l_size), "%.1fMB", (float)u_con->u_file_size/1048576.0);
	else  
		g_snprintf(l_size, sizeof(l_size), "%.1fGB", (float)u_con->u_file_size/1073741824.0);
	
	switch (u_con->state) {
	case GS_WAITINGCONNECT:
		strcpy(l_status, "Connecting...");
		break;
	case GS_FAILED:
		strcpy(l_status, "Failed");
		break;
	default:
		break;
	}	
	
	/*g_snprintf(l_speed, sizeof(l_speed), "%d", u_con->fi->speed);*/
	g_snprintf(l_server, sizeof(l_server), "%s:%d", ip2str(htonl(u_con->ip)), u_con->port);

	gtk_clist_set_text(GTK_CLIST(upload_list), u_con->id-1, 0, file_name);
	gtk_clist_set_text(GTK_CLIST(upload_list), u_con->id-1, 1, l_size);
	gtk_clist_set_text(GTK_CLIST(upload_list), u_con->id-1, 2, l_status);
	/* gtk_clist_set_text(GTK_CLIST(upload_list), u_con->id-1, 3, l_speed); */
	gtk_clist_set_text(GTK_CLIST(upload_list), u_con->id-1, 4, l_server);
 
	/* go to uploads */
	gtk_notebook_set_page(GTK_NOTEBOOK(main_notebook), 2);
}

void gui_update_up_con(struct newtella_connection *u_con)
{
	gchar l_status[255], l_speed[255];
	
	switch (u_con->state) {
	case GS_WAITINGCONNECT:
		strcpy(l_status, "Connecting...");
		break;
	case GS_FAILED:
		strcpy(l_status, "Failed");
		break;
	case GS_CONNECTED:
		g_snprintf(l_status, sizeof(l_status), "%.1f %%", 
			   (float) (u_con->total_sent/(float)u_con->u_file_size)*100.0);
		g_snprintf(l_speed, sizeof(l_speed), "%.1f k/s",
			   (float) (u_con->total_sent/(float)(time((time_t *)NULL)-u_con->last_time)/1000.0));
		break;
	case GS_COMPLETE:
		strcpy(l_status, "Done!");
		break;
	default:
		break;
	}	

	gtk_clist_set_text(GTK_CLIST(upload_list), u_con->id-1, 2, l_status);
	gtk_clist_set_text(GTK_CLIST(upload_list), u_con->id-1, 3, l_speed);
}

void gui_remove_con_from_list(gint row)
{
	gtk_clist_remove(GTK_CLIST(con_list), row);
}

void gui_remove_down_con_from_list(gint row)
{
	gtk_clist_remove(GTK_CLIST(download_list), row);
}

void gui_remove_up_con_from_list(gint row)
{
	gtk_clist_remove(GTK_CLIST(upload_list), row);
}

void gui_reset_cons(void)
{
	g_list_free(gl_con);
	gtk_clist_clear(GTK_CLIST(con_list));
	gl_con = NULL;
	gl_options->current_con = 0;
}

void gui_max_con_changed(void)
{
	gl_options->max_con =
		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(max_con));

	/* sanity check */
	if (gl_options->max_con < gl_options->min_con) {
		gl_options->max_con = gl_options->min_con;
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(max_con), gl_options->max_con);
	}

}

void gui_min_con_changed(void)
{
	gl_options->min_con =
		gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(min_con));

        /* sanity check */
	if (gl_options->max_con < gl_options->min_con) {
		gl_options->min_con = gl_options->max_con;
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(min_con), gl_options->min_con);
	}

}

void gui_toggle_auto_connect(GtkToggleButton *toggle, gpointer data)
{
	gl_options->auto_connect = gtk_toggle_button_get_active(toggle);
}

void gui_toggle_accept_in_con(GtkToggleButton *toggle, gpointer data)
{
	gl_options->accept_in_con = gtk_toggle_button_get_active(toggle);

	if (!gl_options->accept_in_con) 
		newtella_kill_server();
	else {
		if (!newtella_init_server(gl_options->newtella_port)) { 
			gtk_toggle_button_set_active(toggle, FALSE);
		}
	} 	
}

void gui_add_con(void)
{
	guint16 port;
	IP ip;
	char *t, *l, *s = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(host_combo_box)->entry)));
	struct hostent *host;
	GList *temp_list;
	GtkWidget *new_list_item;

 	temp_list = GTK_LIST(GTK_COMBO(host_combo_box)->list)->children;

	/* Search the list for the host just added */
	while (temp_list && temp_list->data)
	{
		/* Get the string from the label */
		gtk_label_get(GTK_LABEL(GTK_BIN(temp_list->data)->child), &l);

		if (!l)
        		continue;
      		if (!g_strcasecmp(l, s))
			break;

		temp_list = temp_list->next;
	}

	if(temp_list == NULL)
	{
		/* The string was not found. 
		   Therefore add it to the list
		*/
		new_list_item = gtk_list_item_new_with_label(s);
		gtk_widget_show(new_list_item);
		gtk_container_add(GTK_CONTAINER(GTK_COMBO(host_combo_box)->list), new_list_item);

	}

	if (strchr(s, ':')) {
		
		t = memchr(s, ':', strlen(s));      
		*t++ = 0;
    
		port = atol(t);
		host = gethostbyname(s);
		if (host != NULL) {
			memcpy(&ip, host->h_addr, 4);
			con_add(ip.dword, port);

		} else
			perror("gethostbyname");

	}

	newtella_free(s);	
}

void gui_button_pressed(GtkWidget *widget, gpointer data)
{
	time_t st;
	char buf[128];

	/* main event handler for all buttons */
	switch ((int)data) {
	case BTN_ADV_CONFIG:
		config_draw();
		break;
	case BTN_STRT_SEARCH:
		gnutella_send_query(g_strdup(gtk_entry_get_text(GTK_ENTRY(search_edit_box))));
		st = time((time_t *) NULL);
		g_snprintf(buf, sizeof(buf), "%s Looking for: %s", ctime(&st),
			   g_strdup(gtk_entry_get_text(GTK_ENTRY(search_edit_box))));
		gtk_label_set_text(GTK_LABEL(status_label), buf);
		break;
	case BTN_ADD_CON:
		gui_add_con();
		break;
	case BTN_HELP:
		break;
	case BTN_CLR_SEARCH:
		gui_clean_found_results();
		break;
	case BTN_RESET_CON:
		gui_reset_cons();
		break;
	case BTN_EXIT: /* shutdown the application */

		/* close the server if it is alive */
		if (gl_options->accept_in_con)
			newtella_kill_server();

		/* shutdown gui */
		gtk_widget_destroy(window);

		/* save config/hosts */
		config_save_to_file();
		host_save_cache_to_file();

		/* bye... */
		gtk_exit(1);
		break;
	default:
		break;
	}
}

void gui_download_file(GtkWidget *gtkclist, gint row, gint col, GdkEventButton *event, 
		       gpointer gdata)
{
	struct file_found *download_file;
	GList *iter;
 
        if (event->type != GDK_2BUTTON_PRESS) 
		return;

	iter = g_list_nth(gl_results, row);
	download_file = iter->data;

	download_add(download_file);
}









