/*-*-linux-c-*-*/

/*
 *  gnewtellium - Newtella for Unix
 *  Copyright (C) 2001 Elias Athanasopoulos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* HTTP header build/parse functions */

int http_header_init(http_header *http)
{
	http->is_response = 0;
	http->code = 0;
	http->range_start = 0;
	http->range_end = 0;
	http->content_length = 0;
	http->content_range_start = 0;
	http->content_range_end = 0;
	http->content_range_total = 0;

	return 1;
}

int http_header_build(http_header *http)
{
	char codestr[MAX_TEMPBUF_LEN];
	char lengthstr[MAX_TEMPBUF_LEN];
	char rangestr[MAX_TEMPBUF_LEN];

	if (http->is_response)
	{
		http->header = "HTTP ";
		sprintf(codestr, "%d", http->code);
	
		http->header += codestr;
		http->header += " ";
		http->header += http->reason;
		http->header += "\r\n";
	
		if (strlen(http->server))
		{
			http->header += "Server: ";
			http->header += http->server;
			http->header += "\r\n";
		}
		if (strlen(http->content_type))
		{
			http->header += "Content-Type: ";
			http->header += http->content_type;
			http->header += "\r\n";
		}
		if (http->content_length)
		{
			http->header += "Content-Length: ";
			sprintf(lengthstr, "%d", http->content_length);
			http->header += lengthstr;
			http->header += "\r\n";
		}
		if (http->content_range_start || http->content_range_end)
		{
			http->header += "Content-Range: ";
			sprintf(rangestr, "bytes=%d-%d/%d", http->content_range_start, 
				http->content_range_end, http->content_range_total);
			http->header += rangestr;
			http->header += "\r\n";
		}
		http->header += "\r\n";
	}
	else
	{
		http->header = http->type;
		http->header += " ";
		http->header += http->file_name;
		http->header += " ";
		http->header += http->version_string;
		http->header += "\r\n";
		if (strlen(http->user_agent))
		{
			http->header += "User-Agent: ";
			http->header += http->user_agent;
			http->header += "\r\n";
		}
		if (strlen(http->range))
		{
			http->header += "Range: ";
			http->header += http->range;
			http->header += "\r\n";
		}
		http->header += "\r\n";
	}

	return(1);
}







